@@ -24,13 +24,14 @@
 
 #include <glib.h>
 #include <startup_config.h>
+#include <libosso.h>
 
 G_BEGIN_DECLS
 
 /* TYPE MACROS */
 #define STARTUP_APP(obj) ((StartupApp *) (obj))
 
-typedef struct _StartupApp        StartupApp;
+typedef struct _StartupApp StartupApp;
 
 typedef void   (startup_app_top_cb)          (StartupApp *app,
                                               gpointer    data);
@@ -39,15 +40,30 @@ typedef void   (startup_app_state_change_cb) (StartupApp *app,
                                               guint       newstate,
                                               gpointer    data);
 
+struct _StartupApp {
+  StartupConfig *config;
+  osso_context_t *osso;
+
+  startup_app_top_cb *top_cb;
+  gpointer top_cb_data;
+  startup_app_state_change_cb *state_change_cb;
+  gpointer state_change_cb_data;
+
+  gchar *service;
+  gchar *path;
+  gchar *iface;
+};
+
+
 /* Game messages
  */
 enum {
   GAME_RUN = 0,
   GAME_CONTINUE,
   GAME_RESTART,
-  GAME_PAUSE,
   GAME_CLOSE,
-  LAST_MESSAGE
+  GAME_SAVE,
+  GAME_LOAD,
 };
 
 /* Game state
@@ -59,17 +75,9 @@ enum {
   GAME_CLOSED,
 };
 
-StartupApp    *startup_app_new             (void);
-void           startup_app_destroy         (StartupApp *app);
-
 gboolean       startup_app_set_config_file (StartupApp  *app,
                                             const gchar *file);
 StartupConfig *startup_app_get_config      (StartupApp  *app);
-gboolean       startup_app_initialize      (StartupApp  *app);
-void           startup_app_deinitialize    (StartupApp  *app);
-gboolean       startup_app_send_game_msg   (StartupApp  *app,
-                                            const guint  msg);
-guint          startup_app_get_game_state  (StartupApp  *app);
 
 void           startup_app_set_top_cb      (StartupApp  *app,
                                             startup_app_top_cb *top_cb,
