@@ -22,67 +22,27 @@
     Boston, MA 02111-1307, USA.
 */
 
-/* Function prototypes */
 #include <gdk/gdkkeysyms.h>
 #include <glib/gstdio.h>
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-helper.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <hildon/hildon-help.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include "callbacks.h"
 #include "state_save.h"
-#include "../debug.h"
+#include "../appdata.h"
 
 #define PAUSE_WITH_HW_KEY "/apps/osso/games_startup/pause_with_hw_key"
 #define ENABLE_KEY_TIMEOUT 1000
 
-void set_save_menu_state(gboolean state);
-
-/* this function is used to check if game's statefile exists or not
- * after the game has been stopped and/or paused to osso-games-launcher.
- *
- * If statefile doesnt exist, game state was not saved and user should be
- * notified of such event since he cannot procede where he was left off */
-
-static void check_game_state(AppUIData *app) {
-    gint gt = SETTINGS_GAMETYPE_UNKNOWN;
-    gchar *gamestate_file = NULL;
-
-    gt = settings_gametype(app);
-
-    /* I'll use switch here since, in the future we might need to check
-     * other gametypes too, not just marbles */
-    switch(gt) {
-        case  SETTINGS_GAMETYPE_MARBLES:
-            gamestate_file = g_build_filename(g_get_home_dir(),
-                                               ".lmarbles_state",
-                                               NULL);
-            break;
-
-        default:
-            break;
-    }
-
-    /* So, now we know what is the statefile that we should have,
-     * lets test it */
-
-    if (gamestate_file != NULL) {
-        if ( !g_file_test(gamestate_file,G_FILE_TEST_IS_REGULAR) ) {
-            hildon_banner_show_information(GTK_WIDGET(app->main_view), NULL, dgettext("ke-recv","cerm_device_memory_full") );
-        }
-        g_free(gamestate_file);
-    }
-}
 
 
 static gboolean
 enable_fullscreen(gpointer data)
 {
-    AppData *appdata = (AppData *) data;
-    appdata->app_ui_data->timeouts = 0;
-    appdata->app_ui_data->got_pause = FALSE;
+    app_data->app_ui_data->timeouts = 0;
+    app_data->app_ui_data->got_pause = FALSE;
     return FALSE;
-
 }
 
 void
@@ -105,37 +65,33 @@ undimm_everything(AppUIData * app)
             gtk_widget_set_sensitive(app->restart_button, TRUE);
         }
     }
-    if (app->plugin->info->plugin_cb)
+    if (app_data->plugin->info->plugin_cb)
     {
-        app->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_PLAYING);
+        app_data->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_PLAYING);
     }
-    return;
 }
 
 void
-dimm_everything(AppData * ad)
+dimm_everything(AppUIData * ad)
 {
-    gtk_widget_set_sensitive(GTK_WIDGET(ad->app_ui_data->main_menu_bar),
+    gtk_widget_set_sensitive(GTK_WIDGET(ad->main_menu_bar),
                              FALSE);
-    if (GTK_WIDGET_VISIBLE(ad->app_ui_data->play_button))
+    if (GTK_WIDGET_VISIBLE(ad->play_button))
     {
-        gtk_widget_set_sensitive(ad->app_ui_data->play_button, FALSE);
+        gtk_widget_set_sensitive(ad->play_button, FALSE);
     }
-    if (GTK_WIDGET_VISIBLE(ad->app_ui_data->continue_button))
+    if (GTK_WIDGET_VISIBLE(ad->continue_button))
     {
-        gtk_widget_set_sensitive(ad->app_ui_data->continue_button, FALSE);
+        gtk_widget_set_sensitive(ad->continue_button, FALSE);
     }
-    if (GTK_WIDGET_VISIBLE(ad->app_ui_data->restart_button))
+    if (GTK_WIDGET_VISIBLE(ad->restart_button))
     {
-        gtk_widget_set_sensitive(ad->app_ui_data->restart_button, FALSE);
+        gtk_widget_set_sensitive(ad->restart_button, FALSE);
     }
-    if (ad->app_ui_data->plugin->info->plugin_cb)
+    if (app_data->plugin->info->plugin_cb)
     {
-        ad->app_ui_data->plugin->info->plugin_cb(NULL,
-                                                 (gpointer)
-                                                 MA_GAME_PLAYING_START);
+        app_data->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_PLAYING_START);
     }
-    return;
 }
 
 void
@@ -146,7 +102,6 @@ save_nomem_response(GtkDialog * dialog, gint arg1, gpointer user_data)
     app_ui_data->nomem_dialog = NULL;
 }
 
-
 gboolean
 dialog_key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
 {
@@ -209,15 +164,12 @@ save_nomem_key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
 
 /* Callback function when a game returns */
 void
-startup_ui_top_cb(AppUIData * app, gpointer data)
+startup_ui_top_cb(StartupApp *app, gpointer data)
 {
-    osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
     AppUIData *ui = (AppUIData *) (data);
     gtk_window_present(GTK_WINDOW(ui->main_view));
 }
 
-
-
 /* The following function is from INDT's games-startup code. The copyrights
  * belongs to them. The INDT's games-startup was released under LGPL license. 
  * This function is modified under LGPL license */
@@ -231,12 +183,12 @@ startup_app_rpc_event_cb(const gchar * interface,
     AppUIData *app;
     GameState oldstate;
     osso_rpc_t val;
-    osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
+    g_debug("%s\n", __FUNCTION__);
     g_return_val_if_fail(data != NULL, OSSO_ERROR);
     g_return_val_if_fail(method != NULL, OSSO_ERROR);
     g_return_val_if_fail(retval != NULL, OSSO_ERROR);
-    app = (AppUIData *) (data);
 
+    app = app_data->app_ui_data;
 
     retval->type = DBUS_TYPE_BOOLEAN;
     retval->value.b = TRUE;
@@ -245,15 +197,14 @@ startup_app_rpc_event_cb(const gchar * interface,
     /* Handle pause method */
     if (g_ascii_strcasecmp(method, GAME_PAUSE_METHOD) == 0)
     {
-        // newstate = GAME_PAUSED;
         int with_hw_key = settings_get_int(PAUSE_WITH_HW_KEY);
-        osso_log(LOG_DEBUG, "%s: Got pause message\n", __FUNCTION__);
+        g_debug("%s: Got pause message\n", __FUNCTION__);
         app->got_pause = TRUE;
         if ((with_hw_key == 286) || (with_hw_key == GDK_F5))
         {
             gtk_window_iconify(GTK_WINDOW(app->main_view));
             settings_set_int(PAUSE_WITH_HW_KEY, 0);
-            osso_log(LOG_DEBUG, "%s: ..with home key\n", __FUNCTION__);
+            g_debug("%s: ..with home key\n", __FUNCTION__);
         }
         else
         {
@@ -262,43 +213,44 @@ startup_app_rpc_event_cb(const gchar * interface,
         set_gamestate(gs_paused);
         if (GTK_WIDGET_VISIBLE(app->play_button))
         {
-            osso_log(LOG_DEBUG,"%s: setting focus to play button\n",__FUNCTION__);
+            g_debug("%s: setting focus to play button\n",__FUNCTION__);
             gtk_widget_grab_focus(app->play_button);
         }
         if (GTK_WIDGET_VISIBLE(app->continue_button))
         {
-            osso_log(LOG_DEBUG,"%s: setting focus continue button\n",__FUNCTION__);
+            g_debug("%s: setting focus continue button\n",__FUNCTION__);
             gtk_widget_grab_focus(app->continue_button);
         }
-        check_game_state(app);
+        if (app_data->plugin->info->plugin_cb)
+        {
+            app_data->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_CHECKSTATE);
+        }
         undimm_everything(app);
     }
     else if (g_ascii_strcasecmp(method, GAME_CLOSE_METHOD) == 0 ||
              g_ascii_strcasecmp(method, GAME_CRASHED_METHOD) == 0 ||
              g_ascii_strcasecmp(method, GAME_END_METHOD) == 0)
     {
-        // newstate = GAME_CLOSED;
-        osso_log(LOG_DEBUG, "%s: Got end message\n", __FUNCTION__);
+        g_debug("%s: Got end message\n", __FUNCTION__);
         int with_hw_key = settings_get_int(PAUSE_WITH_HW_KEY);
         if ((with_hw_key == 286) || (with_hw_key == GDK_F5))
         {
             gtk_window_iconify(GTK_WINDOW(app->main_view));
             settings_set_int(PAUSE_WITH_HW_KEY, 0);
-            osso_log(LOG_DEBUG, "%s: ..with home key\n", __FUNCTION__);
+            g_debug("%s: ..with home key\n", __FUNCTION__);
         }
         set_gamestate(gs_closed);
-        if (app->plugin->info->plugin_cb)
+        if (app_data->plugin->info->plugin_cb)
         {
-            app->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_ENDED);
+            app_data->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_CHECKSTATE);
+            app_data->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_ENDED);
         }
-        check_game_state(app);
         undimm_everything(app);
     }
     else /* Handle home method */
     if (g_ascii_strcasecmp(method, GAME_HOME_METHOD) == 0)
     {
         gtk_window_iconify(GTK_WINDOW(app->main_view));
-        // newstate = GAME_PAUSED;
         set_gamestate(gs_paused);
         if (GTK_WIDGET_VISIBLE(app->play_button))
         {
@@ -308,13 +260,16 @@ startup_app_rpc_event_cb(const gchar * interface,
         {
             gtk_widget_grab_focus(app->continue_button);
         }
-        check_game_state(app);
+        if (app_data->plugin->info->plugin_cb)
+        {
+            app_data->plugin->info->plugin_cb(NULL, (gpointer) MA_GAME_CHECKSTATE);
+        }
     }
     else /* Handle Load method */
     if (g_ascii_strcasecmp(method, GAME_MIME_OPEN) == 0)
     {
         val = g_array_index(args, osso_rpc_t, (guint) 0);
-        osso_log(LOG_DEBUG, "Loading game\n");
+        g_debug("Loading game\n");
         if ((val.type == DBUS_TYPE_STRING) && (val.value.s != NULL))
         {
             game_mime_open((gchar *) val.value.s);
@@ -336,11 +291,10 @@ startup_app_rpc_event_cb(const gchar * interface,
         return OSSO_ERROR;
     }
 
-    /* oldstate = app->state; app->state = newstate; */
-    if (app->state_change_cb)
+    if (app_data->startup_app->state_change_cb)
     {
-        app->state_change_cb(app, oldstate, get_game_state(),
-                             app->state_change_cb_data);
+        app_data->startup_app->state_change_cb(app_data->startup_app, oldstate, get_game_state(),
+                             app_data->startup_app->state_change_cb_data);
     }
     return OSSO_OK;
 }
@@ -353,14 +307,9 @@ startup_app_rpc_event_cb(const gchar * interface,
 void
 startup_app_top_event_cb(const gchar * arguments, gpointer data)
 {
-    AppUIData *app;
-    osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
-
-
-    g_return_if_fail(data != NULL);
-
-    app = (AppUIData *) (data);
-
+    StartupApp *app = (StartupApp *) data;
+    g_return_if_fail(app != NULL);
+    
     if (app->top_cb)
         app->top_cb(app, app->top_cb_data);
 }
@@ -373,10 +322,9 @@ startup_app_top_event_cb(const gchar * arguments, gpointer data)
 void
 startup_ui_game_quit_cb(GtkWidget * widget, gpointer data)
 {
-    osso_log(LOG_DEBUG, "%s", __FUNCTION__);
+    g_debug("%s", __FUNCTION__);
 
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
+    AppUIData *ui = app_data->app_ui_data;
 
     if (ui->timeouts)
     {
@@ -387,21 +335,22 @@ startup_ui_game_quit_cb(GtkWidget * widget, gpointer data)
         set_game_state_async(GAME_CLOSE);
     }
 
-    if (ui->plugin && ui->plugin->info->write_config) {
-        ui->plugin->info->write_config();
+    if (app_data->plugin && app_data->plugin->info->write_config) {
+        app_data->plugin->info->write_config();
     }
 
-    if (ui->plugin && ui->plugin->send_game_close)
+    if (app_data->plugin && app_data->plugin->send_game_close)
     {
         if (get_game_state() == gs_running || get_game_state() == gs_paused)
         {
             set_game_state_async(GAME_CLOSE);
-        } else {
         }
     }
-    if (ui->plugin && ui->plugin->info->unload) {
-        ui->plugin->info->unload();
+    if (app_data->plugin && app_data->plugin->info->unload) {
+        app_data->plugin->info->unload();
     }
+    /* if exiting cleanly, clear state */
+    clear_state();
 }
 
 /* The following function is from INDT's games-startup code. The copyrights
@@ -413,42 +362,38 @@ startup_ui_game_quit_cb(GtkWidget * widget, gpointer data)
 void
 startup_ui_game_restart_cb(GtkWidget * widget, gpointer data)
 {
-    // gboolean answer = FALSE;
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
+    AppUIData *ui = app_data->app_ui_data;
+    
     if (ui->low_memory)
     {
         show_infoprint(__("memr_ib_operation_disabled"));
         return;
     }
-    // AppUIData *ui = data;
-    // if (paused){
     if (get_game_state() == gs_paused)
     {
         gchar *restart_needed = NULL;
         restart_needed =
-            (gchar *) startup_config_read_entry(ui->indt_startup_config,
+            (gchar *) startup_config_read_entry(app_data->startup_app->config,
                                                 "ShowRestartDialog", NULL);
         if (restart_needed)
         {
             if (g_ascii_strncasecmp(restart_needed, "FALSE", 6) == 0)
             {
-                if (ui->plugin->info->plugin_cb)
+                if (app_data->plugin->info->plugin_cb)
                 {
-                    ui->plugin->info->plugin_cb(
+                    app_data->plugin->info->plugin_cb(
                         NULL, (gpointer) MA_GAME_RESTORE_LAST_SETTINGS);
                 }
-                if (ui->plugin && ui->plugin->info->write_config)
-                    ui->plugin->info->write_config();
+                if (app_data->plugin && app_data->plugin->info->write_config)
+                    app_data->plugin->info->write_config();
 
                 g_set_application_name(ui->game_title);
-                if (ui->plugin->info->plugin_cb)
+                if (app_data->plugin->info->plugin_cb)
                 {
-                    ui->plugin->info->plugin_cb(widget,
+                    app_data->plugin->info->plugin_cb(widget,
                                                 (gpointer)
                                                 MA_GAME_PLAYING_START);
                 }
-                // startup_app_send_game_msg (ui->app, GAME_RESTART);
                 set_game_state_async(GAME_RESTART);
                 set_gamestate(gs_running);
                 return;
@@ -456,21 +401,20 @@ startup_ui_game_restart_cb(GtkWidget * widget, gpointer data)
         }
         if (ui_show_restart_game_dialog(ui) == GAME_RESPONSE_OK)
         {
-            if (ui->plugin->info->plugin_cb)
+            if (app_data->plugin->info->plugin_cb)
             {
-               ui->plugin->info->plugin_cb(
+               app_data->plugin->info->plugin_cb(
                     NULL, (gpointer) MA_GAME_RESTORE_LAST_SETTINGS);
             }
-            if (ui->plugin && ui->plugin->info->write_config)
-                ui->plugin->info->write_config();
+            if (app_data->plugin && app_data->plugin->info->write_config)
+                app_data->plugin->info->write_config();
 
             g_set_application_name(ui->game_title);
-            if (ui->plugin->info->plugin_cb)
+            if (app_data->plugin->info->plugin_cb)
             {
-                ui->plugin->info->plugin_cb(widget,
+                app_data->plugin->info->plugin_cb(widget,
                                             (gpointer) MA_GAME_PLAYING_START);
             }
-            // startup_app_send_game_msg (ui->app, GAME_RESTART);
             set_game_state_async(GAME_RESTART);
             set_gamestate(gs_running);
         }
@@ -486,9 +430,7 @@ startup_ui_game_restart_cb(GtkWidget * widget, gpointer data)
 void
 startup_ui_game_play_cb(GtkWidget * widget, gpointer data)
 {
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
-	DMSG("Entering %s\n",__FUNCTION__);
+    AppUIData *ui = app_data->app_ui_data;
     if (ui->low_memory)
     {
         show_infoprint(__("memr_ib_operation_disabled"));
@@ -499,21 +441,21 @@ startup_ui_game_play_cb(GtkWidget * widget, gpointer data)
 	//"New settings will come into effect" if needed, and after that write the configuration
     if (get_game_state() == gs_paused)
     {
-        if (ui->plugin->info->plugin_cb)
+        if (app_data->plugin->info->plugin_cb)
         {
-            ui->plugin->info->plugin_cb(widget,
+            app_data->plugin->info->plugin_cb(widget,
                                         (gpointer)
                                         MA_GAME_RESET_OLD_SETTINGS);
         }
     }
-    if (ui->plugin && ui->plugin->info->write_config)
+    if (app_data->plugin && app_data->plugin->info->write_config)
     {
-        ui->plugin->info->write_config();
+        app_data->plugin->info->write_config();
     }
 
-    if (ui->plugin->info->plugin_cb)
+    if (app_data->plugin->info->plugin_cb)
     {
-        ui->plugin->info->plugin_cb(widget, (gpointer) MA_GAME_PLAYING_START);
+        app_data->plugin->info->plugin_cb(widget, (gpointer) MA_GAME_PLAYING_START);
     }
     if (get_game_state() == gs_paused)
     {
@@ -526,12 +468,11 @@ startup_ui_game_play_cb(GtkWidget * widget, gpointer data)
     set_gamestate(gs_running);
 }
 
-void
+static void
 set_save_menu_state(gboolean state)
 {
-    osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
+    g_debug("%s\n", __FUNCTION__);
+    AppUIData *ui = app_data->app_ui_data;
     if (state == TRUE)
     {                           /* Check, if the file is read-only, or not */
         gchar *file = NULL;
@@ -546,8 +487,8 @@ set_save_menu_state(gboolean state)
             GnomeVFSFileInfo *dir_info = gnome_vfs_file_info_new();
             if (file_info != NULL && dir_info != NULL) {
         
-                osso_log(LOG_DEBUG, "%s  44\n", __FUNCTION__);
-                osso_log(LOG_DEBUG, "Fname: %s\n", file);
+                g_debug("%s  44\n", __FUNCTION__);
+                g_debug("Fname: %s\n", file);
                 
                 vfs_res = file ? gnome_vfs_get_file_info(file, file_info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS) : GNOME_VFS_ERROR_GENERIC;
                 if (vfs_res == GNOME_VFS_OK) {
@@ -595,15 +536,12 @@ set_save_menu_state(gboolean state)
  * This function is modified under LGPL license */
 
 void
-startup_ui_state_change_cb(AppUIData * app,
+startup_ui_state_change_cb(StartupApp * app,
                            guint oldstate, guint newstate, gpointer data)
 {
-    // AppUIData *ui = (AppUIData *) (data);
-    osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
+    g_debug("%s\n", __FUNCTION__);
+    AppUIData *ui = app_data->app_ui_data;
     g_assert(ui);
-    // app changed to ui!!
     switch (newstate)
     {
         case gs_unloaded:
@@ -612,12 +550,12 @@ startup_ui_state_change_cb(AppUIData * app,
 
         case gs_paused:
             ui->timeouts =
-                g_timeout_add(ENABLE_KEY_TIMEOUT, enable_fullscreen, appdata);
+                g_timeout_add(ENABLE_KEY_TIMEOUT, enable_fullscreen, app_data);
             game_state_changed(gs_paused);
-            if (ui->plugin)
-                if (ui->plugin->info)
-                    if (ui->plugin->info->plugin_cb)
-                        ui->plugin->info->plugin_cb(NULL,
+            if (app_data->plugin)
+                if (app_data->plugin->info)
+                    if (app_data->plugin->info->plugin_cb)
+                        app_data->plugin->info->plugin_cb(NULL,
                                                     (gpointer)
                                                     MA_GAME_PLAYING);
 
@@ -627,15 +565,14 @@ startup_ui_state_change_cb(AppUIData * app,
                 set_save_menu_state(TRUE);
                 gtk_widget_set_sensitive(ui->game_saveas_menu, TRUE);
             }
-            if (ui->plugin && ui->plugin->info->update_menu)
-                ui->plugin->info->update_menu();
-            // paused = TRUE;
+            if (app_data->plugin && app_data->plugin->info->update_menu)
+                app_data->plugin->info->update_menu();
             break;
 
         case gs_closed:
             game_state_changed(gs_closed);
-            if (ui->plugin && ui->plugin->info->update_menu)
-                ui->plugin->info->update_menu();
+            if (app_data->plugin && app_data->plugin->info->update_menu)
+                app_data->plugin->info->update_menu();
             break;
 
         default:
@@ -652,9 +589,7 @@ startup_ui_state_change_cb(AppUIData * app,
 void
 startup_ui_menu_action_cb(gpointer data, guint action, GtkWidget * menu_item)
 {
-    // AppUIData *ui = (AppUIData*) (data);
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
+    AppUIData *ui = app_data->app_ui_data;
 
     switch (action)
     {
@@ -667,76 +602,29 @@ startup_ui_menu_action_cb(gpointer data, guint action, GtkWidget * menu_item)
             break;
 
         case MA_GAME_CLOSE:
-            // startup_ui_game_quit_cb (menu_item, ui);
             game_quit_cb(menu_item, ui);
             break;
     }
 }
 
-/**	
-  Private variable for AppUIData, initialized using cb_init()
-  This is used by the insensitive menu callback(s) 
-*/
-static AppUIData *_app_ui = NULL;
-
-GtkTreeModel *
-create_highscore_list(struct HighScore * scores)
-{
-    int i = 0;
-    GtkTreeStore *tree_store;
-    GtkTreeIter iter;
-
-    if (scores == NULL)
-        return NULL;
-
-    /* New tree store with HIGHSCORE_COLS columns, specifying three first of
-     * them to be string */
-    tree_store = gtk_tree_store_new(HIGHSCORE_COLS,
-                                    G_TYPE_STRING,
-                                    G_TYPE_STRING, G_TYPE_STRING);
-
-    /* Building contents for the tree store */
-    for (i = 0; i < HIGHSCORE_SIZE; i++)
-    {
-        if (scores[i].level != NULL /* && scores[i].time!=NULL &&
-                                     * scores[i].date!=NULL */ )
-        {
-            /* New row */
-            gtk_tree_store_append(tree_store, &iter, NULL);
-            /* Setting string values for the specified columns */
-            gtk_tree_store_set(tree_store, &iter,
-                               HIGHSCORE_LEVEL_COLUMN, scores[i].level,
-                               HIGHSCORE_TIME_COLUMN, scores[i].time,
-                               HIGHSCORE_DATE_COLUMN, scores[i].date, -1);
-        }
-    }
-
-    return GTK_TREE_MODEL(tree_store);
-}
-
 void
 menu_set_save_dimmed(AppUIData * app_ui_data, gboolean dimmed)
 {
-    if (app_ui_data->plugin && app_ui_data->plugin->menu_open_save)
+    if (app_data->plugin && app_data->plugin->menu_open_save)
     {
         gtk_widget_set_sensitive(app_ui_data->game_save_menu, dimmed);
         gtk_widget_set_sensitive(app_ui_data->game_saveas_menu, dimmed);
     }
-
 }
 
 gboolean
 game_quit_cb(GtkWidget * widget, gpointer data)
 {
-    AppData *appdata = get_app_data();
-    AppUIData *ui = appdata->app_ui_data;
-
-    if (ui->plugin)
+    if (app_data->plugin)
     {
         startup_ui_game_quit_cb(NULL, NULL);
-        // return;
     }
-    gtk_widget_hide(GTK_WIDGET(get_app_data()->app_ui_data->main_view));
+    gtk_widget_hide(GTK_WIDGET(app_data->app_ui_data->main_view));
     gtk_main_iteration();
     gtk_main_quit();
     return TRUE;
@@ -748,57 +636,41 @@ top_changed(GObject * self, GParamSpec * property_param, gpointer data)
     HildonProgram *program = HILDON_PROGRAM(self);
     if (hildon_program_get_is_topmost(program))
     {
-        osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
-        hildon_program_set_can_hibernate(((AppData *) data)->app_ui_data->app,
+        g_debug("%s\n", __FUNCTION__);
+        hildon_program_set_can_hibernate(app_data->app_ui_data->app,
                                          FALSE);
     }
     else
     {
-        AppData *ad = get_app_data();
         /* Remove menu when app lose focus */
-        gtk_menu_popdown(ad->app_ui_data->main_menu_bar);
-        if (ad->app_ui_data->plugin->info->plugin_cb)
+        gtk_menu_popdown(app_data->app_ui_data->main_menu_bar);
+        if (app_data->plugin->info->plugin_cb)
         {
-            ad->app_ui_data->plugin->info->plugin_cb(NULL,
+            app_data->plugin->info->plugin_cb(NULL,
                                                      (gpointer)
                                                      STARTUP_UNTOP);
         }
-        save_state(ad);
-        hildon_program_set_can_hibernate(ad->app_ui_data->app, TRUE);
+        save_state();
+        hildon_program_set_can_hibernate(app_data->app_ui_data->app, TRUE);
     }
 }
 
 void
 game_play_cb(GtkWidget * widget, gpointer data)
 {
-    AppData *app = get_app_data();
-    if (app->app_ui_data->low_memory)
+    if (app_data->app_ui_data->low_memory)
     {
         show_infoprint(__("memr_ib_operation_disabled"));
         return;
     }
-    GameState gs = get_game_state();
-
-    if (get_game_type() == gt_indt)
-    {
-        startup_ui_game_play_cb(widget, data);
-        return;
-    }
-
-
-    if ((gs == gs_unloaded) || (gs == gs_closed))
-    {
-        game_play();
-        ui_update_status((AppUIData *) data);
-    }
+    startup_ui_game_play_cb(widget, data);
 }
 
 void
 game_continue_cb(GtkWidget * widget, gpointer data)
 {
-    AppData *app = get_app_data();
-    osso_log(LOG_DEBUG, "%s\n", __FUNCTION__);
-    if (app->app_ui_data->low_memory)
+    g_debug("%s\n", __FUNCTION__);
+    if (app_data->app_ui_data->low_memory)
     {
         show_infoprint(__("memr_ib_operation_disabled"));
         return;
@@ -810,8 +682,7 @@ game_continue_cb(GtkWidget * widget, gpointer data)
 void
 game_restart_cb(GtkWidget * widget, gpointer data)
 {
-    AppData *app = get_app_data();
-    if (app->app_ui_data->low_memory)
+    if (app_data->app_ui_data->low_memory)
     {
         show_infoprint(__("memr_ib_operation_disabled"));
         return;
@@ -842,7 +713,6 @@ key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
 
     switch (event->keyval)
     {
-            // case GDK_Return:
         case GDK_F6:
             gs = get_game_state();
             if (!app_ui_data->got_pause)
@@ -850,7 +720,6 @@ key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
                 if ((gs == gs_unloaded) || (gs == gs_closed)
                     || (gs == gs_paused))
                 {
-                    // menu_action(app_ui_data, MA_GAME_PLAY, NULL);
                     startup_ui_game_play_cb(NULL, NULL);
                 }
                 return TRUE;
@@ -860,9 +729,9 @@ key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
             return FALSE;
             break;
         case GDK_F5:
-            if (app_ui_data->plugin->info->plugin_cb)
+            if (app_data->plugin->info->plugin_cb)
             {
-                app_ui_data->plugin->info->plugin_cb(NULL,
+                app_data->plugin->info->plugin_cb(NULL,
                                                      (gpointer)
                                                      MA_GAME_HOME_PRESSED);
             }
@@ -873,21 +742,12 @@ key_press(GtkWidget * widget, GdkEventKey * event, gpointer data)
     return FALSE;
 }
 
-/* Initializes data for callbacks module */
-void
-cb_init(AppUIData * app_ui)
-{
-    /* store app ui data pointer */
-    _app_ui = app_ui;
-}
-
 void
 show_infoprint(gchar * msg)
 {
-    AppData *ad = get_app_data();
-    if (ad != NULL)
+    if (app_data != NULL)
     {
-        hildon_banner_show_information(GTK_WIDGET(ad->app_ui_data->main_view),
+        hildon_banner_show_information(GTK_WIDGET(app_data->app_ui_data->main_view),
                                        NULL, msg);
     }
 }
@@ -896,7 +756,7 @@ show_infoprint(gchar * msg)
 void
 on_insensitive_menu_press(GtkWidget * widget, gchar * message)
 {
-    hildon_banner_show_information(GTK_WIDGET(_app_ui->main_view), NULL,
+    hildon_banner_show_information(GTK_WIDGET(app_data->app_ui_data->main_view), NULL,
                                    message);
 }
 
@@ -908,61 +768,31 @@ on_delete_event(GtkWidget * widget, gpointer data)
 
     game_clean_up_and_quit(widget, data);
 
-    return TRUE;                /* we've handled it (FALSE would _not_ work!) 
-                                 */
+    return TRUE;
 }
 
 void
 game_clean_up_and_quit(GtkWidget * widget, gpointer data)
 {
-    GnomeVFSResult gnomevfs_res;
-    if (gnomevfs_handle != NULL)
-    {
-        gnomevfs_res = gnome_vfs_monitor_cancel(gnomevfs_handle);
-    }
-
     game_quit_cb(widget, data);
 }
 
+/* Handles SIGTERM for background killing */
 void
-volume_unmounted_cb(GnomeVFSVolumeMonitor * vfsvolumemonitor,
-                    GnomeVFSVolume * volume, gpointer user_data)
+on_bgkill(int signal)
 {
-    AppUIData *app_ui_data = (AppUIData *) user_data;
-    gchar *dst_text = NULL;
-    gchar *volume_text = NULL;
-    GnomeVFSURI *dst_uri;
-    gchar *uri_scheme;
+    /* if user is playing (or paused), remember that
+     * we have to continue next time */
 
-    g_return_if_fail(app_ui_data != NULL);
-
-    dst_text = settings_get_string(SETTINGS_CHESS_SAVE_FILE);
-    uri_scheme = gnome_vfs_get_uri_scheme(dst_text);
-    if (!uri_scheme)
-        dst_text = gnome_vfs_get_uri_from_local_path(dst_text);
-    else
-        g_free(uri_scheme);
-    dst_uri = gnome_vfs_uri_new(dst_text);
-    g_free(dst_text);
-
-    dst_text = gnome_vfs_uri_to_string(dst_uri, GNOME_VFS_URI_HIDE_NONE);
-
-    if (dst_text != NULL)
-    {
-        /* the mount point of the changed volume */
-        volume_text = gnome_vfs_volume_get_activation_uri(volume);
+    GameState gs = get_game_state();
+    
+    if (gs == gs_running || gs == gs_paused) {
+	save_state();
     }
 
-    /* check if unmounting this volume has anything to do with us */
-    if (volume_text && dst_text && (g_str_has_prefix(dst_text, volume_text)))
-    {
-        app_ui_data->is_mmc = (app_ui_data->mmc_uri
-                               && g_str_has_prefix(volume_text,
-                                                   app_ui_data->mmc_uri));
-    }
-
-    if (dst_text)
-        g_free(dst_text);
-    if (volume_text)
-        g_free(volume_text);
+    /* quit gracefully */
+    
+    gtk_widget_hide(GTK_WIDGET(app_data->app_ui_data->main_view));
+    gtk_main_quit();
 }
+
