@@ -32,6 +32,7 @@
 #include "dbus.h"
 #include "startup_plugin.h"
 #include "../state_save.h"
+#include "main.h"
 
 #define PLUGIN_HEIGHT           100
 #define PLUGIN_WIDTH             -1
@@ -240,6 +241,34 @@ ui_create_main_app(AppData * app_data)
     return result;
 }
 
+void
+ui_change_play_continue_button(AppData * app_data,ChangeButton next_button)
+{
+    AppUIData *app_ui_data=app_data->app_ui_data;
+    StartupUI *ui = app_data->plugin->gs->ui;
+    GtkWidget *button_show=NULL,*button_hide=NULL;
+    switch (next_button)
+    {
+        case CHANGE_PLAY:
+        {
+            button_hide=app_ui_data->continue_button;
+            button_show=app_ui_data->play_button;
+            break;
+        }
+        case CHANGE_CONTINUE:
+        {
+            button_hide=app_ui_data->play_button;
+            button_show=app_ui_data->continue_button;
+            break;
+        }
+    }
+    gtk_widget_hide(button_hide);
+    gtk_widget_show(button_show);
+    gtk_widget_grab_focus(button_show);
+    gtk_widget_set_sensitive(button_show,GTK_WIDGET_IS_SENSITIVE(button_hide));
+    ui->play_button=button_show;
+}
+
 gboolean
 ui_update_status(AppUIData * app_ui_data)
 {
@@ -258,9 +287,7 @@ ui_update_status(AppUIData * app_ui_data)
         gtk_label_set_text(GTK_LABEL(app_ui_data->controls_state_label),
                            _("game_ia_startup_not_started"));
 
-        gtk_widget_show(app_ui_data->play_button);
-        gtk_widget_grab_focus(app_ui_data->play_button);
-        gtk_widget_hide(app_ui_data->continue_button);
+        ui_change_play_continue_button(get_app_data(),CHANGE_PLAY);
 
         gtk_widget_set_sensitive(app_ui_data->restart_button, FALSE);
         gtk_widget_set_sensitive(app_ui_data->play_button, TRUE);
@@ -274,10 +301,7 @@ ui_update_status(AppUIData * app_ui_data)
 
         gtk_label_set_text(GTK_LABEL(app_ui_data->controls_state_label),
                            _("game_ia_startup_paused"));
-        gtk_widget_hide(app_ui_data->play_button);
-
-        gtk_widget_show(app_ui_data->continue_button);
-        gtk_widget_grab_focus(app_ui_data->continue_button);
+        ui_change_play_continue_button(get_app_data(),CHANGE_CONTINUE);
 
         gtk_widget_set_sensitive(app_ui_data->restart_button, TRUE);
         if (app_ui_data->window_front_needed)
@@ -295,9 +319,7 @@ ui_update_status(AppUIData * app_ui_data)
         gtk_label_set_text(GTK_LABEL(app_ui_data->controls_state_label),
                            _("game_ia_startup_not_started"));
 
-        gtk_widget_show(app_ui_data->play_button);
-        gtk_widget_grab_focus(app_ui_data->play_button);
-        gtk_widget_hide(app_ui_data->continue_button);
+        ui_change_play_continue_button(get_app_data(),CHANGE_PLAY);
 
         gtk_widget_set_sensitive(app_ui_data->restart_button, FALSE);
         gtk_widget_set_sensitive(app_ui_data->play_button, TRUE);
@@ -417,15 +439,11 @@ ui_create_games_common_view(AppUIData * app_ui_data)
     /* Show proper buttons */
     if (gs == gs_unloaded || gs == gs_closed)
     {
-        gtk_widget_show(app_ui_data->play_button);
-        gtk_widget_hide(app_ui_data->continue_button);
-        gtk_widget_grab_focus(app_ui_data->play_button);
+        ui_change_play_continue_button(get_app_data(),CHANGE_PLAY);
     }
     else if (gs == gs_paused)
     {
-        gtk_widget_hide(app_ui_data->play_button);
-        gtk_widget_show(app_ui_data->continue_button);
-        gtk_widget_grab_focus(app_ui_data->continue_button);
+        ui_change_play_continue_button(get_app_data(),CHANGE_CONTINUE);
     }
 
     /* Restart button */
