@@ -42,7 +42,6 @@ typedef struct _StartupUI            StartupUI;
 gboolean startup_init_plugin(StartupPlugin *plugin);
 
 struct _StartupPluginInfo {
-  // FIXME
   
   GtkWidget *  (* load)         (void);
   void         (* unload)       (void);
@@ -99,9 +98,6 @@ struct _StartupPlugin {
         return TRUE; \
     }
 
-StartupPlugin *startup_plugin_load          (const gchar   *path);
-gboolean       startup_plugin_unload        (StartupPlugin *plugin);
-
 G_END_DECLS
 
 #endif
